# This file is part of the Multiphase Python Repository by HZDR.
#
# Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
# Website: https://hzdr.de
#
# Multiphase Python Repository by HZDR is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Multiphase Python Repository by HZDR is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the Multiphase Python Repository by HZDR. If not, see
# <http://www.gnu.org/licenses/>.

"""Test simulation case module."""

from multiphasepy import case
from os import path, remove
from hashlib import sha256


def test_foambasecase():
    """Test simulation case function."""
    assert case.FoamBaseCase("tests/data/case/simple").name() == "simple"


def test_foamextcase_metadata():
    """Test reading of metadata for plain simulation case."""
    assert case.FoamExtCase("tests/data/case/simple")._metadata == {
        "name": "Simple Case",
        "validation": {
            "alpha.air": {
                "experiment": "validation/alpha.dat",
                "reference": "validation/reference/line.csv",
                "simulation": "postProcessing/sampleFields/4/line.csv",
            }
        },
        "keywords": {
            "BUBBLE SIZE": ["monodisperse"],
            "BUBBLE SIZE CHANGES": ["density-change"],
            "COMPRESSIBILITY": ["incompressible"],
        },
    }


def test_foamextcase_metadata_jinja2():
    """Test reading of metadata for simulation case with Jinja2 template."""
    assert case.FoamExtCase(
        "tests/data/case/jinja2", mvalues={"endtime": "4"}
    )._metadata == {
        "name": "Jinja2 Templated Case",
        "validation": {
            "alpha.air": {
                "experiment": "validation/alpha.dat",
                "reference": "validation/reference/line.csv",
                "simulation": "postProcessing/sampleFields/4/line.csv",
            }
        },
    }
    remove("tests/data/case/jinja2/case.yml")


def test_foamcase_controlDict():
    """Test reading of controlDict from simple case."""
    assert case.FoamBaseCase("tests/data/case/simple").get_dictionary(
        "system/controlDict"
    ) == {
        "FoamFile": {
            "format": "ascii",
            "class": "dictionary",
            "object": "controlDict",
        },
        "application": "foamRun",
        "solver": "multiphaseEuler",
        "startFrom": "startTime",
        "startTime": "0",
        "stopAt": "endTime",
        "endTime": "4",
        "deltaT": "0.001",
        "writeControl": "adjustableRunTime",
        "writeInterval": "1",
        "purgeWrite": "0",
        "writeFormat": "ascii",
        "writePrecision": "6",
        "writeCompression": "uncompressed",
        "timeFormat": "general",
        "timePrecision": "6",
        "runTimeModifiable": "yes",
        "adjustTimeStep": "yes",
        "maxCo": "0.5",
        "maxDeltaT": "1",
    }


def test_foamcase_decomposeParDict():
    """Test reading of decomposeParDict from simple case."""
    assert case.FoamBaseCase("tests/data/case/simple").get_dictionary(
        "system/decomposeParDict"
    ) == {
        "FoamFile": {
            "format": "ascii",
            "class": "dictionary",
            "object": "decomposeParDict",
        },
        "numberOfSubdomains": "4",
        "method": "simple",
        "simpleCoeffs": {"n": ["1", "1", "4"]},
    }


def test_checkkeywords():
    """Test for compairing keywords against reference list."""
    tcase = case.FoamExtCase("tests/data/case/simple")
    reference = {
        "BUBBLE SIZE": ["monodisperse", "polydisperse"],
        "BUBBLE SIZE CHANGES": [
            "coalescence-breakup",
            "density-change",
            "fixed-bubble-size",
        ],
        "COMPRESSIBILITY": ["compressible", "incompressible"],
    }

    false_reference = {
        "BUBBLE SIZE": ["polydisperse"],
        "BUBBLE SIZE CHANGES": ["density-change", "fixed-bubble-size"],
        "COMPRESSIBILITY": ["compressible"],
    }

    assert tcase.checkkeywords(reference) is True
    assert tcase.checkkeywords(false_reference) is False


def test_exec():
    """Test execution of commands on a simulation case."""
    tcase = case.FoamBaseCase("tests/data/case/simple")

    # Remove left-over files from previous runs
    try:
        remove("tests/data/case/simple/log.echo")
        remove("tests/data/case/simple/log.ls.banana")
    except FileNotFoundError:
        pass

    # Call the different versions and check for log files
    assert tcase.exec("echo", ["apple is a fruit"]) is True
    assert path.exists("tests/data/case/simple/log.echo") is True

    # Use sha256 as there is a serious warning for MDF5 and SHA1
    # Do not use hashlib.file_digest as it requires python >= 3.11
    h = sha256()
    buf = bytearray(2**18)
    view = memoryview(buf)
    with open("tests/data/case/simple/log.echo", "rb", buffering=0) as f:
        while True:
            size = f.readinto(buf)
            if size == 0:
                break  # EOF
            h.update(view[:size])

    assert (
        h.hexdigest()
        == "4053601a863b4e2e33674674f3d0552c729ffb14ffed51f1e1d9b7d8163b2dfc"
    )

    assert tcase.exec("echo", ["apple is a fruit"]) is False
    assert tcase.exec("echo", ["apple is a fruit"], append=True) is True
    assert tcase.exec("echo", ["apple is a fruit"], overwrite=True) is True
    assert tcase.exec("ls", ["-a", "banana.toml"], suffix="banana") is False
    assert path.exists("tests/data/case/simple/log.ls.banana") is True

    # Use sha256 as there is a serious warning for MDF5 and SHA1
    # Do not use hashlib.file_digest as it requires python >= 3.11
    h = sha256()
    buf = bytearray(2**18)
    view = memoryview(buf)
    with open("tests/data/case/simple/log.ls.banana", "rb", buffering=0) as f:
        while True:
            size = f.readinto(buf)
            if size == 0:
                break  # EOF
            h.update(view[:size])

    assert (
        h.hexdigest()
        == "9e6b50248c9d7e3a20d3e38e7b493f9f003a8f525e8d51e1ee41c4671f020240"
    )

    assert (
        tcase.exec("echo", ["apple is a fruit"], append=True, overwrite=True)
        is False
    )

    # Remove created log files
    remove("tests/data/case/simple/log.echo")
    remove("tests/data/case/simple/log.ls.banana")
