# This file is part of the Multiphase Python Repository by HZDR.
#
# Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
# Website: https://hzdr.de
#
# Multiphase Python Repository by HZDR is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Multiphase Python Repository by HZDR is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the Multiphase Python Repository by HZDR. If not, see
# <http://www.gnu.org/licenses/>.

"""Test the docker module."""

from multiphasepy import docker
from unittest.mock import patch, MagicMock


def test_checkimagename():
    """Test for image naming conventions."""
    assert docker.checkimagename(
        "registry.hzdr.de/group/subgroup/project/image"
    ) == ("registry.hzdr.de", "group/subgroup/project/image", "latest")
    assert docker.checkimagename(
        "registry.hzdr.de/group/subgroup/project/image:tag"
    ) == ("registry.hzdr.de", "group/subgroup/project/image", "tag")
    assert docker.checkimagename(
        "registry.hzdr.de:8000/group/subgroup/project/image:tag"
    ) == ("registry.hzdr.de:8000", "group/subgroup/project/image", "tag")
    assert docker.checkimagename("group/subgroup/project/image:tag") == (
        None,
        "group/subgroup/project/image",
        "tag",
    )
    assert docker.checkimagename(
        "localhost:8000/group/subgroup/project/image:tag"
    ) == (
        "localhost:8000",
        "group/subgroup/project/image",
        "tag",
    )


@patch("os.path.expandvars")
def test_addenvvars(mock_class):
    """Test for adding environment variables."""
    mock_class.return_value = "/test/path/to/dir"

    assert docker.addenvvars(["TESTVAR=/test/path/to/dir"]) == [
        "--env",
        "TESTVAR=/test/path/to/dir",
    ]
    assert docker.addenvvars(["TESTVAR=$ENVVAR"]) == [
        "--env",
        "TESTVAR=/test/path/to/dir",
    ]
    assert docker.addenvvars(["TESTVAR=${ENVVAR}"]) == [
        "--env",
        "TESTVAR=/test/path/to/dir",
    ]
    assert docker.addenvvars(["TESTVAR /test/path/to/dir"]) == []


@patch.dict("os.environ", {"XDG_SESSION_TYPE": "x11", "DISPLAY": ":0"})
@patch("multiphasepy.docker.hostname")
@patch("os.getpid")
@patch("multiphasepy.docker.xauth")
@patch("multiphasepy.docker.xquartz")
@patch("multiphasepy.docker.osname")
def test_adddisplay(mock_uname, mock_xquartz, mock_xauth, mock_pid, mock_host):
    """Test for adding the display options."""
    mock_uname.return_value = "darwin"
    mock_xquartz.return_value = True
    mock_xauth.return_value = True
    mock_pid.return_value = 1
    mock_host.return_value = "banana"
    assert docker.adddisplay() == [
        "--env",
        "DISPLAY=banana:0",
        "--env",
        "XAUTHORITY=/tmp/.docker.xauth.1",
        "--volume",
        "/tmp/.docker.xauth.1:/tmp/.docker.xauth.1",
    ]
    mock_uname.return_value = "linux"
    mock_xquartz.return_value = True
    mock_xauth.return_value = True
    mock_pid.return_value = 1
    mock_host.return_value = "banana"
    assert docker.adddisplay() == [
        "--device",
        "/dev/dri:/dev/dri",
        "--volume",
        "/tmp/.X11-unix:/tmp/.X11-unix",
        "--env",
        "DISPLAY=:0",
        "--env",
        "XAUTHORITY=/tmp/.docker.xauth.1",
        "--volume",
        "/tmp/.docker.xauth.1:/tmp/.docker.xauth.1",
    ]


@patch("shutil.which")
def test_addgpu(mock_which):
    """Test for GPU support options."""
    docker.addgpu()
    mock_which.assert_any_call("nvidia-container-cli")
    mock_which.assert_any_call("nvidia-smi")


@patch.dict("os.environ", {"SSH_AUTH_SOCK": "/run/user/12345/keyring/ssh"})
def test_addssh():
    """Test for SSH support options."""
    assert docker.addssh() == [
        "--env",
        "SSH_AUTH_SOCK=/run/user/12345/keyring/ssh",
        "--volume",
        "/run/user/12345/keyring:/run/user/12345/keyring",
    ]


def test_addvolumes():
    """Test for adding volume mounts with variable expansion."""
    image = MagicMock()
    image.attrs = {"Config": {"Env": ["testvar=/banana/tree"]}}
    assert docker.addvolumes(["/host/path:/container/path"], image) == [
        "--volume",
        "/host/path:/container/path",
    ]

    assert docker.addvolumes(["/host/path:$testvar"], image) == [
        "--volume",
        "/host/path:/banana/tree",
    ]

    assert docker.addvolumes(["/host/path:${testvar}"], image) == [
        "--volume",
        "/host/path:/banana/tree",
    ]


def test_buildimage():
    """Test call to Docker API for building custom image."""
    client = MagicMock()
    client.images.build.return_value = True

    docker.buildimage(client, "image_", "image_b", ["RUN apt-get update"])
    client.images.build.assert_called()


def test_checkrestart():
    """Test for checkrestart function to determine docker command."""
    client = MagicMock()
    container = MagicMock()
    container.configure_mock(attrs={"State": {"Status": "exited"}}, id="54321")
    image = MagicMock()
    image.configure_mock(id="12345")
    client.containers.get.return_value = container
    assert docker.checkrestart(client, "54321") is False
    container.configure_mock(attrs={"State": {"Status": "running"}}, id="54321")
    assert docker.checkrestart(client, "54321") is True
    client.containers.get = MagicMock(side_effect=docker.NotFound("54321"))
    assert docker.checkrestart(client, "54321") is False


@patch("multiphasepy.docker.buildimage")
@patch("multiphasepy.docker.pullimage")
@patch("multiphasepy.docker.checklogin")
def test_getimage(mock_login, mock_pull, mock_build):
    """Test login, pull, build and return of docker images."""
    client = MagicMock()
    # Do not pull and do not build
    docker.getimage(
        client,
        "image",
        False,
        False,
        ["RUN apt-get update"],
        ["/my/test/dir:/my/test/dir"],
        False,
    )
    mock_login.assert_not_called()
    mock_pull.assert_not_called()
    mock_build.assert_not_called()
    # Do pull but skip build (not allowed)
    docker.getimage(
        client,
        "image",
        True,
        False,
        ["RUN apt-get update"],
        ["/my/test/dir:/my/test/dir"],
        False,
    )
    mock_login.assert_not_called()
    mock_pull.assert_not_called()
    mock_build.assert_not_called()
    # Do not pull but build (allowed)
    docker.getimage(
        client,
        "image",
        False,
        True,
        ["RUN apt-get update"],
        ["/my/test/dir:/my/test/dir"],
        False,
    )
    mock_login.assert_not_called()
    mock_pull.assert_not_called()
    mock_build.assert_called_once()
    # Do pull and build (allowed)
    mock_build.reset_mock()
    docker.getimage(
        client,
        "docker.io/image",
        True,
        True,
        ["RUN apt-get update"],
        ["/my/test/dir:/my/test/dir"],
        False,
    )
    mock_login.assert_called_once()
    mock_pull.assert_called_once()
    mock_build.assert_called_once()
    client.images.get.assert_called_with("docker.io/image:latestMPY")


def test_pullimage():
    """Test pulling of images from registry."""
    client = MagicMock()
    docker.pullimage(client, "image")
    client.api.pull.assert_called_once()


def test_cmdargs():
    """Test reading of configuration file."""
    args = docker.cmdargs(
        ["mpydocker", "--config", "tests/data/docker/config.json"]
    )
    assert args.gpu is True
    assert args.ssh is True
    assert args.home is False
    assert (
        args.image
        == "registry.hzdr.de/fwdc/multiphase/code/10-s.2:addonLinux64GccOpt"
    )
    assert args.custom_build == [
        "RUN sudo apt-get update",
        "RUN sudo apt-get install -qy x11-apps",
    ]
    assert args.volume == ["${PWD}:${WM_PROJECT_USER_DIR}/run"]
    assert args.env == ["VARIABLETEST=$PWD"]
    assert args.site is False
    assert args.ccache is True
    assert args.jupyter is False
    assert args.name == "banana"


def test_addjupyter():
    """Test for jupyter options."""
    assert docker.addjupyter() == [
        "-p",
        "8888:8888",
        "--health-cmd",
        "curl -f http://localhost:8888/ || exit 1",
        "--health-interval",
        "5s",
    ]


@patch("multiphasepy.docker.run")
def test_xauth(mock_sub, tmp_path):
    """Test for xauth function."""
    mock_stdout = MagicMock()
    mock_sub.return_value = mock_stdout
    xauthfile = f"{tmp_path}/.docker.xauth.12345"
    # Check for missing display name
    assert docker.xauth("Banana", xauthfile) is False
    # Check if we can run through the full function
    assert docker.xauth("Banana:0", xauthfile) is True
    # Check if we fail, in case grep returns true
    mock_stdout.returncode = 0
    assert docker.xauth("Banana:0", xauthfile) is False
