# This file is part of the Multiphase Python Repository by HZDR.
#
# Copyright (C) 2023 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
# Website: https://hzdr.de
#
# Multiphase Python Repository by HZDR is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Multiphase Python Repository by HZDR is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the Multiphase Python Repository by HZDR. If not, see
# <http://www.gnu.org/licenses/>.

"""Test the foam module."""

from multiphasepy import foam
from hashlib import sha256
from numpy import float64
from pytest import raises


def test_convert_file(tmp_path):
    """Test file converting from dat to csv."""
    infile = "tests/data/case/simple/validation/reference/residuals.dat"
    outfile = tmp_path / "residuals.csv"
    foam.convert(infile, outfile)
    # Use sha256 as there is a serious warning for MDF5 and SHA1
    # Do not use hashlib.file_digest as it requires python >= 3.11
    h = sha256()
    buf = bytearray(2**18)
    view = memoryview(buf)
    with open(outfile, "rb", buffering=0) as f:
        while True:
            size = f.readinto(buf)
            if size == 0:
                break  # EOF
            h.update(view[:size])

    assert (
        h.hexdigest()
        == "038d075cc6ff88b2dfc3c35f2c4ad0641ba2a84f4ea8d566fc3b78c404f89073"
    )


def test_read_data_dat():
    """Test loading of dat file for N/A."""
    data, header = foam.read_data("tests/data/foam/file.dat")
    assert header == ["x", "alphaMean.air"]
    # Test that we can read a N/A as numpy.float64
    assert type(data[0, 1]) is float64
    # Test for mismatch between header length and numpy columns
    with raises(SystemExit):
        foam.read_data("tests/data/foam/file_wrong.dat")


def test_read_probes():
    """Test loading of probes file from simulation software."""
    data, header = foam.read_data(
        "tests/data/case/simple/postProcessing/probes/0/U.water"
    )
    assert header == [
        "Time",
        "Probe 0 (0.5 0.5 0.05) X",
        "Probe 0 (0.5 0.5 0.05) Y",
        "Probe 0 (0.5 0.5 0.05) Z",
        "Probe 1 (0.5 1.5 0.05) X",
        "Probe 1 (0.5 1.5 0.05) Y",
        "Probe 1 (0.5 1.5 0.05) Z",
    ]

    # row, column
    assert data[1, 1] == [0.1]
    assert data[1, 6] == [0]

    data, header = foam.read_data(
        "tests/data/case/simple/postProcessing/probes/0/p_rgh"
    )
    assert header == [
        "Time",
        "Probe 0 (0.0001 1.5707 0.05)",
        "Probe 1 (0.0001 0 0.05)",
    ]

    assert data[0, 1] == -0.0041
    assert data[2, 2] == -0.494


def test_dicttofoam():
    """Test conversion of dictionaries to configuration files."""
    decomposeParDict = {
        "FoamFile": {
            "format": "ascii",
            "class": "dictionary",
            "object": "decomposeParDict",
        },
        "numberOfSubdomains": "4",
        "method": "simple",
        "simpleCoeffs": {"n": ["1", "1", "4"]},
    }

    assert foam.dicttofoam(decomposeParDict) == [
        "FoamFile",
        "{",
        "    format ascii;",
        "    class dictionary;",
        "    object decomposeParDict;",
        "}",
        "numberOfSubdomains 4;",
        "method simple;",
        "simpleCoeffs",
        "{",
        "    n (1 1 4);",
        "}",
    ]
