# This file is part of the Multiphase Python Repository by HZDR.
#
# Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
# Website: https://hzdr.de
#
# Multiphase Python Repository by HZDR is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Multiphase Python Repository by HZDR is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the Multiphase Python Repository by HZDR. If not, see
# <http://www.gnu.org/licenses/>.

"""Test module to test post-processing functions."""

from pytest import approx
import numpy as np
from multiphasepy import post, foam


def test_average():
    """Test for calculating average."""
    default_coordinates = ["Time", "time", "x", "y", "z", "distance"]

    # Check for regular data array with base column first
    data = np.column_stack((np.arange(0, 10), np.arange(0, 10)))
    header = ["Time", "Values"]
    assert post.average(data, header, default_coordinates) == (
        approx([4.5]),
        ["Values"],
    )

    # Check for missing base column
    header = ["Banana", "Values"]
    assert post.average(data, header, default_coordinates) == (None, None)

    # Check for data array with non-regular base column first
    header = ["Banana", "Values"]
    assert post.average(data, header, ["Banana"]) == (approx([4.5]), ["Values"])

    # Check for regular data, but with time column second
    header = ["Values", "time"]
    assert post.average(data, header, default_coordinates) == (
        approx([4.5]),
        ["Values"],
    )

    # Check for data with non constant time stepping
    data = np.array([[1, 0.5], [2.5, -4.2], [3.1, 1], [3.8, 1.1]])
    header = ["Time", "Values"]
    assert post.average(data, header, default_coordinates) == (
        approx([-1.0714285]),
        ["Values"],
    )

    # Test reading of probe file
    data, header = foam.read_data(
        "tests/data/case/simple/postProcessing/probes/0/U.water"
    )
    values, _ = post.average(data, header, default_coordinates)
    assert values == approx([0.1, 0.01, 0, -0.1, 0.01, 0])


def test_gradient():
    """Test for calculating gradient."""
    default_coordinates = ["Time", "time", "x", "y", "z", "distance"]

    # Check for regular data array with coordinate column first
    data = np.column_stack((np.arange(0, 10), np.arange(0, 10)))
    header = ["x", "Value"]
    assert post.gradient(data, header, default_coordinates) == (
        approx(np.column_stack((np.arange(0, 10), np.ones(10)))),
        ["x", "d_Value_d_x"],
    )

    # Check for missing coordinate column
    header = ["Banana", "Value"]
    assert post.gradient(data, header, default_coordinates) == (None, None)

    # Check for data array with non-regular coordinate name
    header = ["Banana", "Value"]
    assert post.gradient(data, header, ["Banana"]) == (
        approx(np.column_stack((np.arange(0, 10), np.ones(10)))),
        ["Banana", "d_Value_d_Banana"],
    )

    # Check for regular data, but with coordinate column second
    data = np.column_stack((np.arange(0, 10), np.arange(0, 10)))
    header = ["Value", "x"]
    assert post.gradient(data, header, default_coordinates) == (
        approx(np.column_stack((np.arange(0, 10), np.ones(10)))),
        ["x", "d_Value_d_x"],
    )

    # Check for multiple values and multiple coordinates
    data = np.column_stack(
        (
            np.arange(0, 10),
            np.arange(0, 20, 2),
            np.ones(10),
            np.arange(0, 10),
        )
    )
    header = ["x", "y", "Value1", "Value2"]
    assert post.gradient(data, header, default_coordinates) == (
        approx(
            np.column_stack(
                (
                    np.arange(0, 10),
                    np.arange(0, 20, 2),
                    np.zeros(10),
                    np.ones(10),
                    np.zeros(10),
                    0.5 * np.ones(10),
                )
            )
        ),
        [
            "x",
            "y",
            "d_Value1_d_x",
            "d_Value2_d_x",
            "d_Value1_d_y",
            "d_Value2_d_y",
        ],
    )

    # Check for data with non constant coordinate stepping
    data = np.array([[1, 0.5], [2.5, -4.2], [3.1, 1], [3.8, 1.1]])
    header = ["Time", "Value"]
    assert post.gradient(data, header, default_coordinates) == (
        approx(
            np.array(
                [
                    [1, -3.1333333],
                    [2.5, 5.2952381],
                    [3.1, 4.7326007],
                    [3.8, 0.1428571],
                ]
            )
        ),
        ["Time", "d_Value_d_Time"],
    )

    # Test reading of probe data
    data, header = foam.read_data(
        "tests/data/case/simple/postProcessing/probes/0/p_rgh"
    )
    assert post.gradient(data, header, default_coordinates) == (
        approx(
            np.array(
                [[0, 0.156, 0.28], [0.025, 0.076, 0.06], [0.05, -0.004, -0.16]]
            )
        ),
        [
            "Time",
            "d_Probe 0 (0.0001 1.5707 0.05)_d_Time",
            "d_Probe 1 (0.0001 0 0.05)_d_Time",
        ],
    )

    # Test reading of graph data
    data, header = foam.read_data(
        "tests/data/case/simple/postProcessing/sampleFields/4/line.csv"
    )
    _, header = post.gradient(data, header, default_coordinates)
    assert header == [
        "x",
        "d_alpha.air_d_x",
        "d_U.water_x_d_x",
        "d_U.water_y_d_x",
        "d_U.water_z_d_x",
        "d_k.water_d_x",
    ]
